@@ -1,3 +1,5 @@
+"""InfluxDBClient is client for API defined in https://github.com/influxdata/influxdb/blob/master/http/swagger.yml."""
+
 from __future__ import absolute_import
 
 import configparser
@@ -16,12 +18,12 @@ from influxdb_client.client.write_api import WriteApi, WriteOptions, PointSettin
 
 
 class InfluxDBClient(object):
+    """InfluxDBClient is client for InfluxDB v2."""
 
     def __init__(self, url, token, debug=None, timeout=10000, enable_gzip=False, org: str = None,
-                 default_tags: dict = None) -> None:
+                 default_tags: dict = None, **kwargs) -> None:
         """
-        :class:`influxdb_client.InfluxDBClient` is client for HTTP API defined
-        in https://github.com/influxdata/influxdb/blob/master/http/swagger.yml.
+        Initialize defaults.
 
         :param url: InfluxDB server API url (ex. http://localhost:9999).
         :param token: auth token
@@ -30,6 +32,9 @@ class InfluxDBClient(object):
         :param enable_gzip: Enable Gzip compression for http requests. Currently only the "Write" and "Query" endpoints
                             supports the Gzip compression.
         :param org: organization name (used as a default in query and write API)
+        :key bool verify_ssl: Set this to false to skip verifying SSL certificate when calling API from https server.
+        :key urllib3.util.retry.Retry retries: Set the default retry strategy that is used for all HTTP requests
+                                               except batching writes. As a default there is no one retry strategy.
 
         """
         self.url = url
@@ -46,16 +51,29 @@ class InfluxDBClient(object):
             conf.host = self.url
         conf.enable_gzip = enable_gzip
         conf.debug = debug
+        conf.verify_ssl = kwargs.get('verify_ssl', True)
 
         auth_token = self.token
         auth_header_name = "Authorization"
         auth_header_value = "Token " + auth_token
 
+        retries = kwargs.get('retries', False)
+
         self.api_client = ApiClient(configuration=conf, header_name=auth_header_name,
-                                    header_value=auth_header_value)
+                                    header_value=auth_header_value, retries=retries)
 
     @classmethod
     def from_config_file(cls, config_file: str = "config.ini", debug=None, enable_gzip=False):
+        """
+        Configure client via '*.ini' file in segment 'influx2'.
+
+        Supported options:
+            - url
+            - org
+            - token
+            - timeout,
+            - verify_ssl
+        """
         config = configparser.ConfigParser()
         config.read(config_file)
 
@@ -72,6 +90,10 @@ class InfluxDBClient(object):
         if config.has_option('influx2', 'org'):
             org = config['influx2']['org']
 
+        verify_ssl = True
+        if config.has_option('influx2', 'verify_ssl'):
+            verify_ssl = config['influx2']['verify_ssl']
+
         default_tags = None
 
         if config.has_section('tags'):
@@ -79,16 +101,28 @@ class InfluxDBClient(object):
 
         if timeout:
             return cls(url, token, debug=debug, timeout=int(timeout), org=org, default_tags=default_tags,
-                       enable_gzip=enable_gzip)
+                       enable_gzip=enable_gzip, verify_ssl=_to_bool(verify_ssl))
 
-        return cls(url, token, debug=debug, org=org, default_tags=default_tags, enable_gzip=enable_gzip)
+        return cls(url, token, debug=debug, org=org, default_tags=default_tags, enable_gzip=enable_gzip,
+                   verify_ssl=_to_bool(verify_ssl))
 
     @classmethod
     def from_env_properties(cls, debug=None, enable_gzip=False):
+        """
+        Configure client via environment properties.
+
+        Supported environment properties:
+            - INFLUXDB_V2_URL
+            - INFLUXDB_V2_ORG
+            - INFLUXDB_V2_TOKEN
+            - INFLUXDB_V2_TIMEOUT
+            - INFLUXDB_V2_VERIFY_SSL
+        """
         url = os.getenv('INFLUXDB_V2_URL', "http://localhost:9999")
         token = os.getenv('INFLUXDB_V2_TOKEN', "my-token")
         timeout = os.getenv('INFLUXDB_V2_TIMEOUT', "10000")
         org = os.getenv('INFLUXDB_V2_ORG', "my-org")
+        verify_ssl = os.getenv('INFLUXDB_V2_VERIFY_SSL', "True")
 
         default_tags = dict()
 
@@ -96,11 +130,12 @@ class InfluxDBClient(object):
             if key.startswith("INFLUXDB_V2_TAG_"):
                 default_tags[key[16:].lower()] = value
 
-        return cls(url, token, debug=debug, timeout=int(timeout), org=org, default_tags=default_tags, enable_gzip=enable_gzip)
+        return cls(url, token, debug=debug, timeout=int(timeout), org=org, default_tags=default_tags,
+                   enable_gzip=enable_gzip, verify_ssl=_to_bool(verify_ssl))
 
     def write_api(self, write_options=WriteOptions(), point_settings=PointSettings()) -> WriteApi:
         """
-        Creates a Write API instance
+        Create a Write API instance.
 
         :param point_settings:
         :param write_options: write api configuration
@@ -110,26 +145,25 @@ class InfluxDBClient(object):
 
     def query_api(self) -> QueryApi:
         """
-        Creates a Query API instance
+        Create a Query API instance.
 
         :return: Query api instance
         """
         return QueryApi(self)
 
     def close(self):
-        """
-        Shutdowns the client
-        """
+        """Shutdown the client."""
         self.__del__()
 
     def __del__(self):
+        """Shutdown the client."""
         if self.api_client:
             self.api_client.__del__()
             self.api_client = None
 
     def buckets_api(self) -> BucketsApi:
         """
-        Creates the Bucket API instance.
+        Create the Bucket API instance.
 
         :return: buckets api
         """
@@ -137,7 +171,7 @@ class InfluxDBClient(object):
 
     def authorizations_api(self) -> AuthorizationsApi:
         """
-        Creates the Authorizations API instance.
+        Create the Authorizations API instance.
 
         :return: authorizations api
         """
@@ -145,7 +179,7 @@ class InfluxDBClient(object):
 
     def users_api(self) -> UsersApi:
         """
-        Creates the Users API instance.
+        Create the Users API instance.
 
         :return: users api
         """
@@ -153,7 +187,7 @@ class InfluxDBClient(object):
 
     def organizations_api(self) -> OrganizationsApi:
         """
-        Creates the Organizations API instance.
+        Create the Organizations API instance.
 
         :return: organizations api
         """
@@ -161,7 +195,7 @@ class InfluxDBClient(object):
 
     def tasks_api(self) -> TasksApi:
         """
-        Creates the Tasks API instance.
+        Create the Tasks API instance.
 
         :return: tasks api
         """
@@ -169,7 +203,7 @@ class InfluxDBClient(object):
 
     def labels_api(self) -> LabelsApi:
         """
-        Creates the Labels API instance.
+        Create the Labels API instance.
 
         :return: labels api
         """
@@ -187,12 +221,11 @@ class InfluxDBClient(object):
             health = health_service.get_health()
             return health
         except Exception as e:
-            print(e)
             return HealthCheck(name="influxdb", message=str(e), status="fail")
 
     def ready(self) -> Ready:
         """
-        Gets The readiness of the InfluxDB 2.0.
+        Get The readiness of the InfluxDB 2.0.
 
         :return: Ready
         """
@@ -201,7 +234,8 @@ class InfluxDBClient(object):
 
     def delete_api(self) -> DeleteApi:
         """
-        Gets the delete metrics API instance
+        Get the delete metrics API instance.
+
         :return: delete api
         """
         return DeleteApi(self)
@@ -240,3 +274,7 @@ class _Configuration(Configuration):
                     return gzip.compress(bytes(_body, "utf-8"))
 
         return _body
+
+
+def _to_bool(verify_ssl):
+    return str(verify_ssl).lower() in ("yes", "true")
