@@ -1,52 +1,64 @@
+"""Point data structure to represent LineProtocol."""
+
+
 import math
 from builtins import int
 from datetime import datetime, timedelta
 from decimal import Decimal
 from numbers import Integral
 
-import ciso8601
 from pytz import UTC
 from six import iteritems
 
+from influxdb_client.client.util.date_utils import get_date_helper
 from influxdb_client.domain.write_precision import WritePrecision
 
 EPOCH = UTC.localize(datetime.utcfromtimestamp(0))
 DEFAULT_WRITE_PRECISION = WritePrecision.NS
-_ESCAPE_KEY = str.maketrans({'\\': '\\\\', ',': r'\,', ' ': r'\ ', '=': r'\=', '\n': ''})
+_ESCAPE_MEASUREMENT = str.maketrans({'\\': '\\\\', ',': r'\,', ' ': r'\ ', '\n': '\\n', '\t': '\\t', '\r': '\\r'})
+_ESCAPE_KEY = str.maketrans({'\\': '\\\\', ',': r'\,', ' ': r'\ ', '=': r'\=', '\n': '\\n', '\t': '\\t', '\r': '\\r'})
 _ESCAPE_STRING = str.maketrans({'\"': r"\"", "\\": r"\\"})
 
 
 class Point(object):
     """
     Point defines the values that will be written to the database.
+
     Ref: http://bit.ly/influxdata-point
     """
 
     @staticmethod
     def measurement(measurement):
+        """Create a new Point with specified measurement name."""
         p = Point(measurement)
         return p
 
     @staticmethod
     def from_dict(dictionary: dict, write_precision: WritePrecision = DEFAULT_WRITE_PRECISION):
+        """Initialize point from 'dict' structure."""
         point = Point(dictionary['measurement'])
-        for tag_key, tag_value in dictionary['tags'].items():
-            point.tag(tag_key, tag_value)
+        if 'tags' in dictionary:
+            for tag_key, tag_value in dictionary['tags'].items():
+                point.tag(tag_key, tag_value)
         for field_key, field_value in dictionary['fields'].items():
             point.field(field_key, field_value)
-        point.time(dictionary['time'], write_precision=write_precision)
+        if 'time' in dictionary:
+            point.time(dictionary['time'], write_precision=write_precision)
         return point
 
     def __init__(self, measurement_name):
+        """Initialize defaults."""
         self._tags = {}
         self._fields = {}
         self._name = measurement_name
         self._time = None
         self._write_precision = DEFAULT_WRITE_PRECISION
+        pass
 
     def time(self, time, write_precision=DEFAULT_WRITE_PRECISION):
         """
         Specify timestamp for DataPoint with declared precision.
+
         If time doesn't have specified timezone we assume that timezone is UTC.
 
         Examples::
@@ -65,15 +77,18 @@ class Point(object):
         return self
 
     def tag(self, key, value):
+        """Add tag with key and value."""
         self._tags[key] = value
         return self
 
     def field(self, field, value):
+        """Add field with key and value."""
         self._fields[field] = value
         return self
 
     def to_line_protocol(self):
-        _measurement = _escape_key(self._name)
+        """Create LineProtocol."""
+        _measurement = _escape_key(self._name, _ESCAPE_MEASUREMENT)
         _tags = _append_tags(self._tags)
         _fields = _append_fields(self._fields)
         if not _fields:
@@ -82,6 +97,11 @@ class Point(object):
 
         return f"{_measurement}{_tags}{_fields}{_time}"
 
+    @property
+    def write_precision(self):
+        """Get precision."""
+        return self._write_precision
+
 
 def _append_tags(tags):
     _return = []
@@ -127,8 +147,10 @@ def _append_time(time, write_precision):
     return f" {int(_convert_timestamp(time, write_precision))}"
 
 
-def _escape_key(tag):
-    return str(tag).translate(_ESCAPE_KEY)
+def _escape_key(tag, escape_list=None):
+    if escape_list is None:
+        escape_list = _ESCAPE_KEY
+    return str(tag).translate(escape_list)
 
 
 def _escape_tag_value(value):
@@ -143,20 +165,23 @@ def _escape_string(value):
 
 
 def _convert_timestamp(timestamp, precision=DEFAULT_WRITE_PRECISION):
+    date_helper = get_date_helper()
     if isinstance(timestamp, Integral):
         return timestamp  # assume precision is correct if timestamp is int
 
     if isinstance(timestamp, str):
-        timestamp = ciso8601.parse_datetime(timestamp)
+        timestamp = date_helper.parse_date(timestamp)
 
     if isinstance(timestamp, timedelta) or isinstance(timestamp, datetime):
 
         if isinstance(timestamp, datetime):
             if not timestamp.tzinfo:
                 timestamp = UTC.localize(timestamp)
-            ns = (timestamp - EPOCH).total_seconds() * 1e9
-        else:
-            ns = timestamp.total_seconds() * 1e9
+            else:
+                timestamp = timestamp.astimezone(UTC)
+            timestamp = timestamp - EPOCH
+
+        ns = date_helper.to_nanoseconds(timestamp)
 
         if precision is None or precision == WritePrecision.NS:
             return ns
@@ -167,9 +192,4 @@ def _convert_timestamp(timestamp, precision=DEFAULT_WRITE_PRECISION):
         elif precision == WritePrecision.S:
             return ns / 1e9
 
-        # elif precision == 'm':
-        #     return ns / 1e9 / 60
-        # elif precision == 'h':
-        #     return ns / 1e9 / 3600
-
     raise ValueError(timestamp)
