@@ -1,11 +1,14 @@
+"""Collect and write time series data to InfluxDB Cloud and InfluxDB OSS."""
+
 # coding: utf-8
 import logging
 import os
+from collections import defaultdict
 from datetime import timedelta
 from enum import Enum
 from random import random
 from time import sleep
-from typing import Union, List
+from typing import Union, List, Any
 
 import rx
 from rx import operators as ops, Observable
@@ -13,28 +16,34 @@ from rx.scheduler import ThreadPoolScheduler
 from rx.subject import Subject
 
 from influxdb_client import WritePrecision, WriteService
-from influxdb_client.client.abstract_client import AbstractClient
+from influxdb_client.client.write.dataframe_serializer import data_frame_to_list_of_points
 from influxdb_client.client.write.point import Point, DEFAULT_WRITE_PRECISION
-from influxdb_client.rest import ApiException
+from influxdb_client.client.write.retry import WritesRetry
 
 logger = logging.getLogger(__name__)
 
 
 class WriteType(Enum):
+    """Configuration which type of writes will client use."""
+
     batching = 1
     asynchronous = 2
     synchronous = 3
 
 
 class WriteOptions(object):
+    """Write configuration."""
 
     def __init__(self, write_type: WriteType = WriteType.batching,
                  batch_size=1_000, flush_interval=1_000,
                  jitter_interval=0,
-                 retry_interval=1_000,
+                 retry_interval=5_000,
+                 max_retries=3,
+                 max_retry_delay=180_000,
+                 exponential_base=5,
                  write_scheduler=ThreadPoolScheduler(max_workers=1)) -> None:
         """
-        Creates write api configuration.
+        Create write api configuration.
 
         :param write_type: methods of write (batching, asynchronous, synchronous)
         :param batch_size: the number of data point to collect in batch
@@ -42,6 +51,10 @@ class WriteOptions(object):
         :param jitter_interval: this is primarily to avoid large write spikes for users running a large number of
                client instances ie, a jitter of 5s and flush duration 10s means flushes will happen every 10-15s.
         :param retry_interval: the time to wait before retry unsuccessful write
+        :param max_retries: the number of max retries when write fails
+        :param max_retry_delay: the maximum delay between each retry attempt in milliseconds
+        :param exponential_base: base for the exponential retry delay, the next delay is computed as
+                                 `retry_interval * exponential_base^(attempts-1) + random(jitter_interval)`
         :param write_scheduler:
         """
         self.write_type = write_type
@@ -49,15 +62,30 @@ class WriteOptions(object):
         self.flush_interval = flush_interval
         self.jitter_interval = jitter_interval
         self.retry_interval = retry_interval
+        self.max_retries = max_retries
+        self.max_retry_delay = max_retry_delay
+        self.exponential_base = exponential_base
         self.write_scheduler = write_scheduler
 
+    def to_retry_strategy(self):
+        """Create a Retry strategy from write options."""
+        return WritesRetry(
+            total=self.max_retries,
+            backoff_factor=self.retry_interval / 1_000,
+            jitter_interval=self.jitter_interval / 1_000,
+            max_retry_delay=self.max_retry_delay / 1_000,
+            exponential_base=self.exponential_base,
+            method_whitelist=["POST"])
+
     def __getstate__(self):
+        """Return a dict of attributes that you want to pickle."""
         state = self.__dict__.copy()
         # Remove write scheduler
         del state['write_scheduler']
         return state
 
     def __setstate__(self, state):
+        """Set your object with the provided dict."""
         self.__dict__.update(state)
         # Init default write Scheduler
         self.write_scheduler = ThreadPoolScheduler(max_workers=1)
@@ -68,15 +96,14 @@ ASYNCHRONOUS = WriteOptions(write_type=WriteType.asynchronous)
 
 
 class PointSettings(object):
+    """Settings to store default tags."""
 
     def __init__(self, **default_tags) -> None:
-
         """
-        Creates point settings for write api.
+        Create point settings for write api.
 
         :param default_tags: Default tags which will be added to each point written by api.
         """
-
         self.defaultTags = dict()
 
         for key, val in default_tags.items():
@@ -91,6 +118,7 @@ class PointSettings(object):
         return value
 
     def add_default_tag(self, key, value) -> None:
+        """Add new default tag with key and value."""
         self.defaultTags[key] = self._get_value(value)
 
 
@@ -118,7 +146,7 @@ class _BatchItemKey(object):
 
     def __eq__(self, o: object) -> bool:
         return isinstance(o, self.__class__) \
-               and self.bucket == o.bucket and self.org == o.org and self.precision == o.precision
+            and self.bucket == o.bucket and self.org == o.org and self.precision == o.precision
 
     def __str__(self) -> str:
         return '_BatchItemKey[bucket:\'{}\', org:\'{}\', precision:\'{}\']' \
@@ -140,10 +168,12 @@ def _body_reduce(batch_items):
     return b'\n'.join(map(lambda batch_item: batch_item.data, batch_items))
 
 
-class WriteApi(AbstractClient):
+class WriteApi:
+    """Implementation for '/api/v2/write' endpoint."""
 
     def __init__(self, influxdb_client, write_options: WriteOptions = WriteOptions(),
                  point_settings: PointSettings = PointSettings()) -> None:
+        """Initialize defaults."""
         self._influxdb_client = influxdb_client
         self._write_service = WriteService(influxdb_client.api_client)
         self._write_options = write_options
@@ -172,7 +202,7 @@ class WriteApi(AbstractClient):
                     ops.merge_all())),
                 # Write data into InfluxDB (possibility to retry if its fail)
                 ops.filter(lambda batch: batch.size > 0),
-                ops.map(mapper=lambda batch: self._retryable(data=batch, delay=self._jitter_delay())),
+                ops.map(mapper=lambda batch: self._to_response(data=batch, delay=self._jitter_delay())),
                 ops.merge_all()) \
                 .subscribe(self._on_next, self._on_error, self._on_complete)
 
@@ -183,48 +213,54 @@ class WriteApi(AbstractClient):
     def write(self, bucket: str, org: str = None,
               record: Union[
                   str, List['str'], Point, List['Point'], dict, List['dict'], bytes, List['bytes'], Observable] = None,
-              write_precision: WritePrecision = DEFAULT_WRITE_PRECISION, **kwargs) -> None:
+              write_precision: WritePrecision = DEFAULT_WRITE_PRECISION, **kwargs) -> Any:
         """
-        Writes time-series data into influxdb.
+        Write time-series data into InfluxDB.
 
-        :param str org: specifies the destination organization for writes; take either the ID or Name interchangeably; if both orgID and org are specified, org takes precedence. (required)
+        :param str org: specifies the destination organization for writes; take either the ID or Name interchangeably;
+                        if both orgID and org are specified, org takes precedence. (required)
         :param str bucket: specifies the destination bucket for writes (required)
-        :param WritePrecision write_precision: specifies the precision for the unix timestamps within the body line-protocol
+        :param WritePrecision write_precision: specifies the precision for the unix timestamps within
+                                               the body line-protocol. The precision specified on a Point has precedes
+                                               and is use for write.
         :param record: Points, line protocol, Pandas DataFrame, RxPY Observable to write
-        :param data_frame_measurement_name: name of measurement for writing Pandas DataFrame
-        :param data_frame_tag_columns: list of DataFrame columns which are tags, rest columns will be fields
-
+        :key data_frame_measurement_name: name of measurement for writing Pandas DataFrame
+        :key data_frame_tag_columns: list of DataFrame columns which are tags, rest columns will be fields
         """
-
         if org is None:
             org = self._influxdb_client.org
 
         if self._point_settings.defaultTags and record is not None:
             for key, val in self._point_settings.defaultTags.items():
-                if isinstance(record, dict):
-                    record.get("tags")[key] = val
-                else:
-                    for r in record:
-                        if isinstance(r, dict):
-                            r.get("tags")[key] = val
-                        elif isinstance(r, Point):
-                            r.tag(key, val)
+                self._append_default_tag(key, val, record)
 
         if self._write_options.write_type is WriteType.batching:
             return self._write_batching(bucket, org, record,
                                         write_precision, **kwargs)
 
-        final_string = self._serialize(record, write_precision, **kwargs)
+        payloads = defaultdict(list)
+        self._serialize(record, write_precision, payloads, **kwargs)
 
         _async_req = True if self._write_options.write_type == WriteType.asynchronous else False
 
-        return self._post_write(_async_req, bucket, org, final_string, write_precision)
+        def write_payload(payload):
+            final_string = b'\n'.join(payload[1])
+            return self._post_write(_async_req, bucket, org, final_string, payload[0])
+
+        results = list(map(write_payload, payloads.items()))
+        if not _async_req:
+            return None
+        elif len(results) == 1:
+            return results[0]
+        return results
 
     def flush(self):
+        """Flush data."""
         # TODO
         pass
 
     def __del__(self):
+        """Close WriteApi."""
         if self._subject:
             self._subject.on_completed()
             self._subject.dispose()
@@ -238,37 +274,32 @@ class WriteApi(AbstractClient):
             self._disposable = None
         pass
 
-    def _serialize(self, record, write_precision, **kwargs) -> bytes:
-        _result = b''
+    def _serialize(self, record, write_precision, payload, **kwargs):
         if isinstance(record, bytes):
-            _result = record
+            payload[write_precision].append(record)
 
         elif isinstance(record, str):
-            _result = record.encode("utf-8")
+            self._serialize(record.encode("utf-8"), write_precision, payload, **kwargs)
 
         elif isinstance(record, Point):
-            _result = self._serialize(record.to_line_protocol(), write_precision, **kwargs)
+            self._serialize(record.to_line_protocol(), record.write_precision, payload, **kwargs)
 
         elif isinstance(record, dict):
-            _result = self._serialize(Point.from_dict(record, write_precision=write_precision),
-                                      write_precision, **kwargs)
+            self._serialize(Point.from_dict(record, write_precision=write_precision),
+                            write_precision, payload, **kwargs)
         elif 'DataFrame' in type(record).__name__:
-            _result = self._serialize(self._data_frame_to_list_of_points(record,
-                                                                         precision=write_precision, **kwargs),
-                                      write_precision,
-                                      **kwargs)
+            _data = data_frame_to_list_of_points(record, self._point_settings, **kwargs)
+            self._serialize(_data, write_precision, payload, **kwargs)
 
         elif isinstance(record, list):
-            _result = b'\n'.join([self._serialize(item, write_precision,
-                                                  **kwargs) for item in record])
-
-        return _result
+            for item in record:
+                self._serialize(item, write_precision, payload, **kwargs)
 
     def _write_batching(self, bucket, org, data,
                         precision=DEFAULT_WRITE_PRECISION,
                         **kwargs):
-        _key = _BatchItemKey(bucket, org, precision)
         if isinstance(data, bytes):
+            _key = _BatchItemKey(bucket, org, precision)
             self._subject.on_next(_BatchItem(key=_key, data=data))
 
         elif isinstance(data, str):
@@ -276,15 +307,14 @@ class WriteApi(AbstractClient):
                                  precision, **kwargs)
 
         elif isinstance(data, Point):
-            self._write_batching(bucket, org, data.to_line_protocol(),
-                                 precision, **kwargs)
+            self._write_batching(bucket, org, data.to_line_protocol(), data.write_precision, **kwargs)
 
         elif isinstance(data, dict):
             self._write_batching(bucket, org, Point.from_dict(data, write_precision=precision),
                                  precision, **kwargs)
 
         elif 'DataFrame' in type(data).__name__:
-            self._write_batching(bucket, org, self._data_frame_to_list_of_points(data, precision, **kwargs),
+            self._write_batching(bucket, org, data_frame_to_list_of_points(data, self._point_settings, **kwargs),
                                  precision, **kwargs)
 
         elif isinstance(data, list):
@@ -297,63 +327,41 @@ class WriteApi(AbstractClient):
 
         return None
 
-    def _data_frame_to_list_of_points(self, data_frame, precision, **kwargs):
-        from ..extras import pd
-        if not isinstance(data_frame, pd.DataFrame):
-            raise TypeError('Must be DataFrame, but type was: {0}.'
-                            .format(type(data_frame)))
-
-        if 'data_frame_measurement_name' not in kwargs:
-            raise TypeError('"data_frame_measurement_name" is a Required Argument')
-
-        if isinstance(data_frame.index, pd.PeriodIndex):
-            data_frame.index = data_frame.index.to_timestamp()
-        else:
-            data_frame.index = pd.to_datetime(data_frame.index)
-
-        if data_frame.index.tzinfo is None:
-            data_frame.index = data_frame.index.tz_localize('UTC')
-
-        data = []
-
-        for c, (row) in enumerate(data_frame.values):
-            point = Point(measurement_name=kwargs.get('data_frame_measurement_name'))
-
-            for count, (value) in enumerate(row):
-                column = data_frame.columns[count]
-                data_frame_tag_columns = kwargs.get('data_frame_tag_columns')
-                if data_frame_tag_columns and column in data_frame_tag_columns:
-                    point.tag(column, value)
-                else:
-                    point.field(column, value)
-
-            point.time(data_frame.index[c], precision)
-
-            if self._point_settings.defaultTags:
-                for key, val in self._point_settings.defaultTags.items():
-                    point.tag(key, val)
-
-            data.append(point)
-
-        return data
+    def _append_default_tag(self, key, val, record):
+        if isinstance(record, Point):
+            record.tag(key, val)
+        elif isinstance(record, dict):
+            record.get("tags")[key] = val
+        elif isinstance(record, list):
+            for item in record:
+                self._append_default_tag(key, val, item)
 
     def _http(self, batch_item: _BatchItem):
 
         logger.debug("Write time series data into InfluxDB: %s", batch_item)
 
+        retry = WritesRetry(
+            total=self._write_options.max_retries,
+            backoff_factor=self._write_options.retry_interval / 1_000,
+            jitter_interval=self._write_options.jitter_interval / 1_000,
+            max_retry_delay=self._write_options.max_retry_delay / 1_000,
+            method_whitelist=["POST"])
+
         self._post_write(False, batch_item.key.bucket, batch_item.key.org, batch_item.data,
-                         batch_item.key.precision)
+                         batch_item.key.precision, urlopen_kw={'retries': retry})
 
         logger.debug("Write request finished %s", batch_item)
 
         return _BatchResponse(data=batch_item)
 
-    def _post_write(self, _async_req, bucket, org, body, precision):
+    def _post_write(self, _async_req, bucket, org, body, precision, **kwargs):
+
         return self._write_service.post_write(org=org, bucket=bucket, body=body, precision=precision,
                                               async_req=_async_req, content_encoding="identity",
-                                              content_type="text/plain; charset=utf-8")
+                                              content_type="text/plain; charset=utf-8",
+                                              **kwargs)
 
-    def _retryable(self, data: str, delay: timedelta):
+    def _to_response(self, data: _BatchItem, delay: timedelta):
 
         return rx.of(data).pipe(
             ops.subscribe_on(self._write_options.write_scheduler),
@@ -361,20 +369,10 @@ class WriteApi(AbstractClient):
             ops.delay(duetime=delay, scheduler=self._write_options.write_scheduler),
             # invoke http call
             ops.map(lambda x: self._http(x)),
-            # if there is an error than retry
-            ops.catch(handler=lambda exception, source: self._retry_handler(exception, source, data)),
+            # catch exception to fail batch response
+            ops.catch(handler=lambda exception, source: rx.just(_BatchResponse(exception=exception, data=data))),
         )
 
-    def _retry_handler(self, exception, source, data):
-
-        if isinstance(exception, ApiException):
-
-            if exception.status == 429 or exception.status == 503:
-                _delay = self._jitter_delay() + timedelta(milliseconds=self._write_options.retry_interval)
-                return self._retryable(data, delay=_delay)
-
-        return rx.just(_BatchResponse(exception=exception, data=data))
-
     def _jitter_delay(self):
         return timedelta(milliseconds=random() * self._write_options.jitter_interval)
 
@@ -394,6 +392,7 @@ class WriteApi(AbstractClient):
         logger.info("the batching processor was disposed")
 
     def __getstate__(self):
+        """Return a dict of attributes that you want to pickle."""
         state = self.__dict__.copy()
         # Remove rx
         del state['_subject']
@@ -402,6 +401,7 @@ class WriteApi(AbstractClient):
         return state
 
     def __setstate__(self, state):
+        """Set your object with the provided dict."""
         self.__dict__.update(state)
         # Init Rx
         self.__init__(self._influxdb_client, self._write_options, self._point_settings)
