@@ -15,10 +15,6 @@ class QueryDataFrameApi(BaseTest):
 
     def setUp(self) -> None:
         super(QueryDataFrameApi, self).setUp()
-        # https://github.com/gabrielfalcao/HTTPretty/issues/368
-        import warnings
-        warnings.filterwarnings("ignore", category=ResourceWarning, message="unclosed.*")
-        warnings.filterwarnings("ignore", category=PendingDeprecationWarning, message="isAlive*")
 
         httpretty.enable()
         httpretty.reset()
