@@ -239,6 +239,7 @@ class TasksApiTest(BaseTest):
         members = self.tasks_api.get_members(task_id=task.id)
         self.assertEqual(len(members), 0)
 
+    @pytest.mark.skip(reason="https://github.com/influxdata/influxdb/issues/19234")
     def test_owner(self):
         task = self.tasks_api.create_task_cron(self.generate_name("it_task"), TASK_FLUX, "0 2 * * *",
                                                self.organization.id)
@@ -284,7 +285,6 @@ class TasksApiTest(BaseTest):
         self.assertIsNone(run.requested_at)
         self.assertIsNotNone(run.links)
 
-        self.assertEqual(run.links.logs, "/api/v2/tasks/" + task.id + "/runs/" + run.id + "/logs")
         self.assertEqual(run.links.retry, "/api/v2/tasks/" + task.id + "/runs/" + run.id + "/retry")
         self.assertEqual(run.links._self, "/api/v2/tasks/" + task.id + "/runs/" + run.id)
         self.assertEqual(run.links.task, "/api/v2/tasks/" + task.id)
