@@ -5,10 +5,12 @@ from __future__ import absolute_import
 import datetime
 import os
 import unittest
-import time
 from datetime import timedelta
 from multiprocessing.pool import ApplyResult
 
+import httpretty
+
+import influxdb_client
 from influxdb_client import Point, WritePrecision, InfluxDBClient
 from influxdb_client.client.write_api import SYNCHRONOUS, ASYNCHRONOUS, PointSettings
 from influxdb_client.rest import ApiException
@@ -42,8 +44,8 @@ class SynchronousWriteTest(BaseTest):
         record = "h2o_feet,location=coyote_creek level\\ water_level=1.0 1"
         self.write_client.write(bucket.name, self.org, record)
 
-        result = self.query_api.query(
-            "from(bucket:\"" + bucket.name + "\") |> range(start: 1970-01-01T00:00:00.000000001Z) |> last()", self.org)
+        result = self.query_api.query(f"from(bucket:\"{bucket.name}\") |> range(start: 1970-01-01T00:00:00.000000001Z)",
+                                      self.org)
 
         self.assertEqual(len(result), 1)
         self.assertEqual(result[0].records[0].get_measurement(), "h2o_feet")
@@ -213,7 +215,7 @@ class SynchronousWriteTest(BaseTest):
         self.write_client.write(_bucket.name, self.org, _bytes)
 
         result = self.query_api.query(
-            "from(bucket:\"" + _bucket.name + "\") |> range(start: 1970-01-01T00:00:00.000000001Z) |> last()", self.org)
+            f"from(bucket:\"{_bucket.name}\") |> range(start: 1970-01-01T00:00:00.000000001Z)", self.org)
 
         self.assertEqual(len(result), 1)
         self.assertEqual(result[0].records[0].get_measurement(), "h2o_feet")
@@ -231,15 +233,15 @@ class SynchronousWriteTest(BaseTest):
         bucket = self.create_test_bucket()
 
         now = pd.Timestamp('1970-01-01 00:00+00:00')
-        data_frame = pd.DataFrame(data=[["coyote_creek", 1.0], ["coyote_creek", 2.0]],
+        data_frame = pd.DataFrame(data=[["coyote_creek", 1], ["coyote_creek", 2]],
                                   index=[now + timedelta(hours=1), now + timedelta(hours=2)],
-                                  columns=["location", "water_level"])
+                                  columns=["location", "water water_level"])
 
         self.write_client.write(bucket.name, record=data_frame, data_frame_measurement_name='h2o_feet',
                                 data_frame_tag_columns=['location'])
 
         result = self.query_api.query(
-            "from(bucket:\"" + bucket.name + "\") |> range(start: 1970-01-01T00:00:00.000000001Z)", self.org)
+            f"from(bucket:\"{bucket.name}\") |> range(start: 1970-01-01T00:00:00.000000001Z)", self.org)
 
         self.assertEqual(1, len(result))
         self.assertEqual(2, len(result[0].records))
@@ -247,14 +249,14 @@ class SynchronousWriteTest(BaseTest):
         self.assertEqual(result[0].records[0].get_measurement(), "h2o_feet")
         self.assertEqual(result[0].records[0].get_value(), 1.0)
         self.assertEqual(result[0].records[0].values.get("location"), "coyote_creek")
-        self.assertEqual(result[0].records[0].get_field(), "water_level")
+        self.assertEqual(result[0].records[0].get_field(), "water water_level")
         self.assertEqual(result[0].records[0].get_time(),
                          datetime.datetime(1970, 1, 1, 1, 0, tzinfo=datetime.timezone.utc))
 
         self.assertEqual(result[0].records[1].get_measurement(), "h2o_feet")
         self.assertEqual(result[0].records[1].get_value(), 2.0)
         self.assertEqual(result[0].records[1].values.get("location"), "coyote_creek")
-        self.assertEqual(result[0].records[1].get_field(), "water_level")
+        self.assertEqual(result[0].records[1].get_field(), "water water_level")
         self.assertEqual(result[0].records[1].get_time(),
                          datetime.datetime(1970, 1, 1, 2, 0, tzinfo=datetime.timezone.utc))
 
@@ -283,7 +285,7 @@ class SynchronousWriteTest(BaseTest):
         self.write_client.write(bucket.name, record=record)
 
         result = self.query_api.query(
-            "from(bucket:\"" + bucket.name + "\") |> range(start: 1970-01-01T00:00:00.000000001Z) |> last()", self.org)
+            f"from(bucket:\"{bucket.name}\") |> range(start: 1970-01-01T00:00:00.000000001Z)", self.org)
 
         self.assertEqual(len(result), 1)
         self.assertEqual(result[0].records[0].get_measurement(), "h2o_feet")
@@ -296,7 +298,7 @@ class SynchronousWriteTest(BaseTest):
         bucket = self.create_test_bucket()
 
         with self.assertRaises(ApiException) as cm:
-            self.write_client.write(bucket.name)
+            self.write_client.write(bucket.name, record="")
         exception = cm.exception
 
         self.assertEqual(400, exception.status)
@@ -307,6 +309,158 @@ class SynchronousWriteTest(BaseTest):
 
         self.assertEqual(len(result), 0)
 
+    def test_write_point_different_precision(self):
+        bucket = self.create_test_bucket()
+
+        point1 = Point('test_precision') \
+            .field('power', 10) \
+            .tag('powerFlow', 'low') \
+            .time(datetime.datetime(2020, 4, 20, 6, 30, tzinfo=datetime.timezone.utc), WritePrecision.S)
+
+        point2 = Point('test_precision') \
+            .field('power', 20) \
+            .tag('powerFlow', 'high') \
+            .time(datetime.datetime(2020, 4, 20, 5, 30, tzinfo=datetime.timezone.utc), WritePrecision.MS)
+
+        writer = self.client.write_api(write_options=SYNCHRONOUS)
+        writer.write(bucket.name, self.org, [point1, point2])
+
+        result = self.query_api.query(
+            f"from(bucket:\"{bucket.name}\") |> range(start: 1970-01-01T00:00:00.000000001Z) |> last() "
+            "|> sort(columns: [\"_time\"], desc: false)", self.org)
+
+        self.assertEqual(len(result), 2)
+        self.assertEqual(len(result[0].records), 1)
+        self.assertEqual(len(result[1].records), 1)
+        self.assertEqual(result[0].records[0].get_time(),
+                         datetime.datetime(2020, 4, 20, 5, 30, tzinfo=datetime.timezone.utc))
+        self.assertEqual(result[1].records[0].get_time(),
+                         datetime.datetime(2020, 4, 20, 6, 30, tzinfo=datetime.timezone.utc))
+
+    def test_write_point_with_default_tags(self):
+        bucket = self.create_test_bucket()
+
+        point = Point("h2o_feet")\
+            .field("water_level", 1)\
+            .tag("location", "creek level")
+
+        self.write_client.write(bucket.name, self.org, point)
+
+        flux_result = self.client.query_api().query(f'from(bucket:"{bucket.name}") |> range(start: 1970-01-01T00:00:00.000000001Z)')
+        self.assertEqual(1, len(flux_result))
+
+        record = flux_result[0].records[0]
+
+        self.assertEqual(self.id_tag, record["id"])
+        self.assertEqual(self.customer_tag, record["customer"])
+        self.assertEqual("LA", record[self.data_center_key])
+
+    def test_write_list_of_list_point_with_default_tags(self):
+        bucket = self.create_test_bucket()
+
+        point = Point("h2o_feet")\
+            .field("water_level", 1)\
+            .tag("location", "creek level")
+
+        self.write_client.write(bucket.name, self.org, [[point]])
+
+        flux_result = self.client.query_api().query(f'from(bucket:"{bucket.name}") |> range(start: 1970-01-01T00:00:00.000000001Z)')
+        self.assertEqual(1, len(flux_result))
+
+        record = flux_result[0].records[0]
+
+        self.assertEqual(self.id_tag, record["id"])
+        self.assertEqual(self.customer_tag, record["customer"])
+        self.assertEqual("LA", record[self.data_center_key])
+
+    def test_check_write_permission_by_empty_data(self):
+        client = InfluxDBClient(url="http://localhost:9999", token="my-token-wrong", org="my-org")
+        write_api = client.write_api(write_options=SYNCHRONOUS)
+
+        with self.assertRaises(ApiException) as cm:
+            write_api.write("my-bucket", self.org, b'')
+        exception = cm.exception
+
+        self.assertEqual(401, exception.status)
+        self.assertEqual("Unauthorized", exception.reason)
+
+        client.__del__()
+
+    def test_write_query_data_nanoseconds(self):
+
+        from influxdb_client.client.util.date_utils_pandas import PandasDateTimeHelper
+        import influxdb_client.client.util.date_utils as date_utils
+
+        date_utils.date_helper = PandasDateTimeHelper()
+
+        bucket = self.create_test_bucket()
+
+        point = Point("h2o_feet") \
+            .field("water_level", 155) \
+            .tag("location", "creek level")\
+            .time('1996-02-25T21:20:00.001001231Z')
+
+        self.write_client.write(bucket.name, self.org, [point])
+
+        flux_result = self.client.query_api().query(
+            f'from(bucket:"{bucket.name}") |> range(start: 1970-01-01T00:00:00.000000001Z)')
+        self.assertEqual(1, len(flux_result))
+
+        record = flux_result[0].records[0]
+
+        self.assertEqual(self.id_tag, record["id"])
+        self.assertEqual(record["_value"], 155)
+        self.assertEqual(record["location"], "creek level")
+        self.assertEqual(record["_time"].year, 1996)
+        self.assertEqual(record["_time"].month, 2)
+        self.assertEqual(record["_time"].day, 25)
+        self.assertEqual(record["_time"].hour, 21)
+        self.assertEqual(record["_time"].minute, 20)
+        self.assertEqual(record["_time"].second, 00)
+        self.assertEqual(record["_time"].microsecond, 1001)
+        self.assertEqual(record["_time"].nanosecond, 231)
+
+        date_utils.date_helper = None
+
+
+class WriteApiTestMock(BaseTest):
+
+    def setUp(self) -> None:
+        httpretty.enable()
+        httpretty.reset()
+
+        conf = influxdb_client.configuration.Configuration()
+        conf.host = "http://localhost"
+        conf.debug = False
+
+        self.influxdb_client = InfluxDBClient(url=conf.host, token="my-token")
+
+    def tearDown(self) -> None:
+        self.influxdb_client.__del__()
+        httpretty.disable()
+
+    def test_writes_synchronous_without_retry(self):
+        httpretty.register_uri(httpretty.POST, uri="http://localhost/api/v2/write", status=503)
+
+        self.write_client = self.influxdb_client.write_api(write_options=SYNCHRONOUS)
+        with self.assertRaises(ApiException) as cm:
+            self.write_client.write("my-bucket", "my-org", "h2o_feet,location=coyote_creek water_level=1.0 1")
+        exception = cm.exception
+
+        self.assertEqual("Service Unavailable", exception.reason)
+        self.assertEqual(1, len(httpretty.httpretty.latest_requests))
+
+    def test_writes_asynchronous_without_retry(self):
+        httpretty.register_uri(httpretty.POST, uri="http://localhost/api/v2/write", status=503)
+
+        self.write_client = self.influxdb_client.write_api(write_options=ASYNCHRONOUS)
+        with self.assertRaises(ApiException) as cm:
+            self.write_client.write("my-bucket", "my-org", "h2o_feet,location=coyote_creek water_level=1.0 1").get()
+        exception = cm.exception
+
+        self.assertEqual("Service Unavailable", exception.reason)
+        self.assertEqual(1, len(httpretty.httpretty.latest_requests))
+
 
 class AsynchronousWriteTest(BaseTest):
 
@@ -382,9 +536,9 @@ class AsynchronousWriteTest(BaseTest):
         bucket = self.create_test_bucket()
 
         _point1 = {"measurement": "h2o_feet", "tags": {"location": "coyote_creek"},
-                       "time": "2009-11-10T22:00:00Z", "fields": {"water_level": 1.0}}
+                   "time": "2009-11-10T22:00:00Z", "fields": {"water_level": 1.0}}
         _point2 = {"measurement": "h2o_feet", "tags": {"location": "coyote_creek"},
-                       "time": "2009-11-10T23:00:00Z", "fields": {"water_level": 2.0}}
+                   "time": "2009-11-10T23:00:00Z", "fields": {"water_level": 2.0}}
 
         _point_list = [_point1, _point2]
 
@@ -425,7 +579,7 @@ class AsynchronousWriteTest(BaseTest):
                                   columns=["location", "water_level"])
 
         async_result = self.write_client.write(bucket.name, record=data_frame, data_frame_measurement_name='h2o_feet',
-                                data_frame_tag_columns=['location'])
+                                               data_frame_tag_columns=['location'])
         async_result.get()
 
         query = 'from(bucket:"' + bucket.name + '") |> range(start: 1970-01-01T00:00:00.000000001Z)'
@@ -488,6 +642,37 @@ class AsynchronousWriteTest(BaseTest):
 
         self.delete_test_bucket(bucket)
 
+    def test_write_point_different_precision(self):
+        bucket = self.create_test_bucket()
+
+        _point1 = Point("h2o_feet").tag("location", "coyote_creek").field("level water_level", 5.0).time(5,
+                                                                                                         WritePrecision.S)
+        _point2 = Point("h2o_feet").tag("location", "coyote_creek").field("level water_level", 6.0).time(6,
+                                                                                                         WritePrecision.US)
+
+        _point_list = [_point1, _point2]
+
+        async_results = self.write_client.write(bucket.name, self.org, _point_list)
+        self.assertEqual(2, len(async_results))
+        for async_result in async_results:
+            async_result.get()
+
+        query = f'from(bucket:"{bucket.name}") |> range(start: 1970-01-01T00:00:00.000000001Z) '\
+                '|> sort(columns: [\"_time\"], desc: false)'
+
+        flux_result = self.client.query_api().query(query)
+
+        self.assertEqual(1, len(flux_result))
+
+        records = flux_result[0].records
+
+        self.assertEqual(2, len(records))
+        self.assertEqual(records[0].get_time(),
+                         datetime.datetime(1970, 1, 1, 0, 0, 0, 6, tzinfo=datetime.timezone.utc))
+        self.assertEqual(records[1].get_time(),
+                         datetime.datetime(1970, 1, 1, 0, 0, 5, tzinfo=datetime.timezone.utc))
+
+
 
 class PointSettingTest(BaseTest):
 
